<?php
/**
 * Ensure's that a user is logged in before using the application
 *
 */
class User_Plugin_Acl extends Zend_Controller_Plugin_Abstract
{
    /**
     * @var String
     */
    const DEFAULT_ROLE = 'guest';

    /**
     * @var String
     */
    const ACCESS_DENIED_ROUTE = 'access-denied';

    /**
     * Authenticate all requests to the service
     *
     * @param Zend_Controller_Request_Abstract $request
     */
    public function preDispatch(Zend_Controller_Request_Abstract $request)
    {
        $acl = Zend_Registry::get('acl');
        $auth = Zend_Auth::getInstance();
        $role = self::DEFAULT_ROLE;
        $resource = $request->getModuleName() . ':' . $request->getControllerName();
        $flashMessenger = Zend_Controller_Action_HelperBroker::getStaticHelper('flashMessenger');
        $redirector = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
        
        if ($auth->hasIdentity()) {
            $authObj = $auth->getStorage()->read();
            $role = $authObj->role;
            
            $view = Zend_Controller_Front::getInstance()->getParam('bootstrap')->getResource('view');
            $view->navigation()->setRole($auth->getIdentity()->role);
            
            $view->role = $auth->getIdentity()->role;
            
            if ($acl->has($resource) &&
                !$acl->isAllowed($role, $resource, $request->getActionName()))
            {
                if ($request->getModuleName() == 'user' && $request->getControllerName() == 'index'
                    && $request->getActionName() == 'access-denied') {
                    return TRUE;
                }

                $flashMessenger->addMessage(
                    'You do not have the proper access level to view that page'
                );
                $redirector->setCode(303)
                           ->setExit(true)
                           ->setGotoRoute(array(), self::ACCESS_DENIED_ROUTE, true);
            }
        } else {
            if ($request->getModuleName() == 'user' && $request->getControllerName() == 'index'
                && $request->getActionName() == 'login') {
                return TRUE;
            }
            $redirector->setCode(303)
                       ->setExit(true)
                       ->setGotoRoute(array(), 'login', true);
        }
    }
}